@@ -8,10 +8,6 @@
 #include <asm/page.h>
 #endif
 
-#ifdef HAVE_SYS_SWAP_H
-#include <sys/swap.h>
-#endif
-
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -20,6 +16,15 @@
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
+#ifdef HAVE_SYS_SWAP_H
+#include <sys/swap.h>
+#endif
+
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
@@ -36,7 +41,13 @@
 #define SWAPD_DEF_CONFIG "swapd.conf"
 #endif
 
+#ifndef SWAPD_SWAP_PERMS
+#define SWAPD_SWAP_PERMS 0600
+#endif
+
 extern char *optarg;
+
+/* Reasonable defaults */
 int64_t minfree = (10 * 1024 *1024), maxfree = 0;
 size_t swapsize = (32 * 1024 * 1024);
 char *swapdir = "/var/tmp";
@@ -79,6 +90,7 @@ int swapd_init_stats(void) {
 	/* Initialize the libstatgrab interface. */
 	if (sg_init() != 0) {
 		/* Failure. */
+		PRINTERR("Unable to initialize statistics interface (libstatgrab).");
 		return(0);
 	}
 #endif
@@ -117,6 +129,11 @@ int64_t swapd_get_free_mem(void) {
 	uint64_t meminfo_part = 0, meminfo = 0;
 
 	meminfofd = fopen("/proc/meminfo", "r");
+
+	if (meminfofd == NULL) {
+		return(-1);
+	}
+
 	while (1) {
 		fgets(buf, sizeof(buf) - 1, meminfofd);
 		if (feof(meminfofd)) {
@@ -148,6 +165,7 @@ int64_t swapd_get_free_mem(void) {
 }
 
 int swapd_call_mkswap(const char *swapfile) {
+#ifdef SWAPD_NEED_MKSWAP
 	char cmdline[8192] = {0};
 	int fd;
 
@@ -158,17 +176,29 @@ int swapd_call_mkswap(const char *swapfile) {
 	fd = open(swapfile, O_RDONLY);
 	fsync(fd);
 	close(fd);
+#endif
 
 	return(0);
 }
 
 int swapd_swapon(const char *swapfile) {
 	int swaponret = -1;
-
-#ifdef SWAPON_TAKES_2_ARGS
+#ifdef HAVE_SWAPON
+#  ifdef SWAPON_TAKES_2_ARGS
+	/* Linux */
 	swaponret = swapon(swapfile, 0);
-#else
+#  else
+	/* BSD */
 	swaponret = swapon(swapfile);
+#  endif
+#else
+# ifdef SWAPD_SWAPON_CMDLINE
+	char cmdline[1024] = {0};
+	snprintf(cmdline, sizeof(cmdline) - 1, SWAPD_SWAPON_CMDLINE " \"%s\"", swapfile);
+	swaponret = system(cmdline);
+#  else
+#    warning Dont know how to swapon() on this platform!
+#  endif
 #endif
 
 	return(swaponret);
@@ -209,6 +239,9 @@ swap_t *swapd_mkswap(const char *swapdir, size_t swapsize, const char *pathname)
 		}
 		close(swapfd);
 
+		/* Set permissions */
+		chmod(swapfilepath, SWAPD_SWAP_PERMS);
+
 		/* Initialize the swap signature. */
 		swapd_call_mkswap(swapfilepath);
 	} else {
@@ -240,23 +273,30 @@ swap_t *swapd_mkswap(const char *swapdir, size_t swapsize, const char *pathname)
 }
 
 int swapd_swapoff(swap_t *swapfile) {
-#ifdef HAVE_SWAPOFF
 	int swapoffret = -1;
+#ifdef HAVE_SWAPOFF
 
 	if (swapfile == NULL) {
 		return(-1);
 	}
 
 	swapoffret = swapoff(swapfile->pathname);
+#else
+#  ifdef SWAPD_SWAPOFF_CMDLINE
+	char cmdline[1024] = {0};
+	snprintf(cmdline, sizeof(cmdline) - 1, SWAPD_SWAPOFF_CMDLINE " \"%s\"", swapfile->pathname);
+	swapoffret = system(cmdline);
+#  else
+#    warning Dont know how to swapoff on this platform
+#  endif
+#endif
 
-	if (swapoffret < 0) {
+	if (swapoffret >= 0) {
 		swapfile->active = 0;
 		return(0);
 	}
+
 	return(-1);
-#else
-	return(0);
-#endif
 }
 
 int swapd_cleanup(const char *swapdir) {
@@ -277,6 +317,7 @@ int swapd_cleanup(const char *swapdir) {
 		}
 		if (strlen(dent->d_name) == 11 && strncmp(dent->d_name, ".swap", 5) == 0) {
 			SPOTVAR_S(dent->d_name);
+			unlink(dent->d_name);
 		}
 	}
 
@@ -401,8 +442,12 @@ int main(int argc, char **argv) {
 	int active_swaps = 0, inactive_swaps = 0, dbg_cmp_active_swaps;
 	int swapoffret = -1;
 	int retval = EXIT_SUCCESS;
+	int gfm_errorcount = 0;
 
 	if (!swapd_init_stats()) {
+#ifdef HAVE_LIBSTATGRAB
+#else
+#endif
 		return(EXIT_FAILURE);
 	}
 
@@ -479,8 +524,13 @@ int main(int argc, char **argv) {
 
 		freemem = swapd_get_free_mem();
 		if (freemem == -1) {
+			gfm_errorcount++;
+			if (gfm_errorcount == 10) {
+				exit(EXIT_FAILURE);
+			}
 			continue;
 		}
+		gfm_errorcount = 0;
 
 		SPOTVAR_LL(freemem);
 
@@ -519,6 +569,8 @@ int main(int argc, char **argv) {
 
 		if (freemem > maxfree && maxfree > 0) {
 			SPOTVAR_I(active_swaps);
+
+			/* Deactivate swaps if there is too much free memory. */
 			if (active_swaps > 0) {
 				for (i = 0; i < (sizeof(swaps) / sizeof(swaps[0])); i++) {
 					if (swaps[i] != NULL) {
@@ -560,7 +612,6 @@ int main(int argc, char **argv) {
 			}
 #endif
 
-
 			continue;
 		}
 
